//
// LSMRegex
//
#include <windows.h>
#include "LSMRegex.h"
#include "LSRegex.h"
#pragma comment(lib,"lsmodule.lib")

// The mandatory pre-setup function.  Our name is "LSMRegex", and the class is LSMRegex.
// This sets up a "ModulePath" variable which contains the path to this module in case we want it,
// and a "PluginLog" variable, which contains the path and filename of what we should use for our
// debug logging if we need it.  It also sets up a variable "pModule" which is the pointer to
// our instanced class.
LSModulePreSetup("Regex",LSMRegex);

// Basic LavishScript datatypes, these get retrieved on startup by our initialize function, so we can
// use them in our Top-Level Objects or custom datatypes
LSType *pStringType=0;
LSType *pIntType=0;
LSType *pBoolType=0;
LSType *pFloatType=0;
LSType *pTimeType=0;
LSType *pByteType=0;
LSType *pIntPtrType=0;
LSType *pBoolPtrType=0;
LSType *pFloatPtrType=0;
LSType *pBytePtrType=0;

LSTypeDefinition *pRegexType=0;

LSInterface *pLSInterface=0;

// Forward declarations of callbacks
//int __cdecl CMD_Regex(int argc, char *argv[]);
//bool __cdecl TLO_Regex(int argc, char *argv[], LSTYPEVAR &Dest);


// The constructor of our class.  General initialization cannot be done yet, because we're not given
// the pointer to the Inner Space interface until it is ready for us to initialize.  Just set the
// pointer we have to the interface to 0.  Initialize data members, too.
LSMRegex::LSMRegex(void)
{
}

// Free any remaining resources in the destructor.  This is called when the DLL is unloaded, but
// Inner Space calls the "Shutdown" function first.  Most if not all of the shutdown process should
// be done in Shutdown.
LSMRegex::~LSMRegex(void)
{
}

// Initialize is called by Inner Space when the extension should initialize.
bool LSMRegex::Initialize(LSInterface *p_LSInterface)
{
	pLSInterface=p_LSInterface;

	// retrieve basic ISData types
	pStringType=pLSInterface->FindLSType("string");
	pIntType=pLSInterface->FindLSType("int");
	pBoolType=pLSInterface->FindLSType("bool");
	pFloatType=pLSInterface->FindLSType("float");
	pTimeType=pLSInterface->FindLSType("time");
	pByteType=pLSInterface->FindLSType("byte");
	pIntPtrType=pLSInterface->FindLSType("intptr");
	pBoolPtrType=pLSInterface->FindLSType("boolptr");
	pFloatPtrType=pLSInterface->FindLSType("floatptr");
	pBytePtrType=pLSInterface->FindLSType("byteptr");

	RegisterCommands();
	RegisterAliases();
	RegisterDataTypes();
	RegisterTopLevelObjects();

	return true;
}

// shutdown sequence
void LSMRegex::Shutdown()
{
	UnRegisterTopLevelObjects();
	UnRegisterDataTypes();
	UnRegisterAliases();
	UnRegisterCommands();
}



void LSMRegex::RegisterCommands()
{
	// add any commands
//	pLSInterface->AddCommand("Regex",CMD_Regex,true,false);
}

void LSMRegex::RegisterAliases()
{
	// add any aliases
}

void LSMRegex::RegisterDataTypes()
{
	// add any datatypes
	// pMyType = new MyType;
	// pLSInterface->AddLSType(*pMyType);
	pRegexType = new LSRegexType;
	pLSInterface->AddLSType(*pRegexType);
}

void LSMRegex::RegisterTopLevelObjects()
{
	// add any Top-Level Objects
	//pLSInterface->AddTopLevelObject("Regex",TLO_Regex);
}

void LSMRegex::UnRegisterCommands()
{
	// remove commands
//	pLSInterface->RemoveCommand("Regex");
}
void LSMRegex::UnRegisterAliases()
{
	// remove aliases
}
void LSMRegex::UnRegisterDataTypes()
{
	// remove data types
	//pLSInterface->RemoveLSType(*pMyType);
	//delete pMyType;

	pLSInterface->RemoveLSTypeDefinition(*pRegexType);
	delete pRegexType;// = new LSRegexType;

}
void LSMRegex::UnRegisterTopLevelObjects()
{
	// remove Top-Level Objects
//	pLSInterface->RemoveTopLevelObject("Regex");
}

void LSMRegex::Pulse()
{
	// Anything to be processed once per "frame".  Frames are not at precise intervals, and
	// are product-dependent.  For example, Inner Space frames are dependent on the framerate
	// of the game.
}

void LSMRegex::AddPreprocessorDefinitions()
{
	pLSInterface->AddPreprocessorDefinition("LSMREGEX","1.00");
}

/*
// Our command, which does pretty much nothing.  It displays a message, and returns
// the success code.  Note: The first argument passed is the full name of the command.
int __cdecl CMD_Regex(int argc, char *argv[])
{
	printf("Regex Command");
	return 0;
}

// Our Top-Level Object, a string that says Hello World.  Used like so: ${Regex}
// Note: Unlike commands, the full name of the TLO is NOT an argument
bool __cdecl TLO_Regex(int argc, char *argv[], LSTYPEVAR &Dest)
{
	Dest.Ptr="Hello World";
	Dest.Type=pStringType;
	return true;
}
/**/


