#pragma once
#include "regexpr2.h"
using namespace regex;

class LSRegex
{
public:
	LSRegex()
	{
		pPattern=0;
		pMatch=0;
		pSplit=0;
		pSubst=0;
		Substitution=0;
		Pattern=0;
	}

	~LSRegex()
	{
		Clear();
	}

	void Clear()
	{
		if (Substitution)
		{
			free(Substitution);
			Substitution=0;
		}
		if (Pattern)
		{
			free(Pattern);
			Pattern=0;
		}
		if (pSubst)
		{
			delete pSubst;
			pSubst=0;
		}
		if (pPattern)
		{
			delete pPattern;
			pPattern=0;
		}
		if (pMatch)
		{
			delete pMatch;
			pMatch=0;
		}
		if (pSplit)
		{
			delete pSplit;
			pSplit=0;
		}
	}

	void SetPattern(int argc, char *argv[],regex::REGEX_FLAGS Flags=NOFLAGS)
	{
		Clear();
		if (!argc)
			return;

		Pattern=strdup(argv[0]);

		for (int arg = 1 ; arg < argc ; arg++)
		{
			if (!stricmp(argv[arg],"GLOBAL"))
				Flags|=GLOBAL;
			else if (!stricmp(argv[arg],"MULTILINE"))
				Flags|=MULTILINE;
			else if (!stricmp(argv[arg],"SINGLELINE"))
				Flags|=SINGLELINE;
			else if (!stricmp(argv[arg],"EXTENDED"))
				Flags|=EXTENDED;
			else if (!stricmp(argv[arg],"RIGHTMOST"))
				Flags|=RIGHTMOST;
			else if (!stricmp(argv[arg],"NOBACKREFS"))
				Flags|=NOBACKREFS;
			else if (!stricmp(argv[arg],"ALLBACKREFS"))
				Flags|=ALLBACKREFS;
			else if (!stricmp(argv[arg],"FIRSTBACKREFS"))
				Flags|=FIRSTBACKREFS;
			else if (!stricmp(argv[arg],"NORMALIZE"))
				Flags|=NORMALIZE;
			else
			{
				if (arg==1)
					Substitution=strdup(argv[arg]);
				else
				{
					// error
				}
			}
		}
		m_Flags=Flags;
	}

	void SetSubstitution(const char *Text)
	{
		if (pSubst)
		{
			delete pSubst;
			pSubst=0;
		}
		if (Substitution)
		{
			free(Substitution);
		}
		if (!Text)
		{
			Substitution=0;
			return;
		}
		Substitution=strdup(Text);
	}

	inline bool PerformSubstitution(string &str)
	{
		if (!Substitution || !Pattern)
			return false;
		if (!pSubst)
			pSubst = new regex::rpattern(Pattern,Substitution,m_Flags,MODE_SAFE);
		subst_results results;
		pSubst->substitute(str,results);
		return true;
	}

	inline bool Match(const char *text)
	{
		if (pMatch)
			delete pMatch;
		if (!Pattern)
		{
			pMatch=0;
			return false;
		}
		if (!pPattern)
			pPattern = new regex::rpattern(Pattern,m_Flags,MODE_SAFE);
		pMatch = new regex::match_results;
		pPattern->match(text,*pMatch);
		/*
		match_results::backref_type br=pPattern->match(text,*pMatch);
		if (br.matched)
		{
			for(int i = 0; i< pMatch->cbackrefs(); i++ )
			{
				br = pMatch->backref(i);
				string str2(br.str());
				printf("%s",str2.c_str());
			}	
		}
		/**/
		return true;
	}
	inline bool Split(const char *text)
	{
		if (pSplit)
			delete pSplit;
		if (!Pattern)
		{
			pSplit=0;
			return false;
		}
		if (!pPattern)
			pPattern = new regex::rpattern(Pattern,m_Flags,MODE_SAFE);

		pSplit = new regex::split_results;
		pPattern->split(text,*pSplit);
		return true;
	}

	char *Pattern;
	char *Substitution;
	regex::REGEX_FLAGS m_Flags;

	regex::rpattern *pSubst;
	regex::rpattern *pPattern;
	regex::match_results *pMatch; 
	regex::split_results *pSplit;
};


class LSRegexType : public LSTypeDefinition
{
public:
	// Set up enumeration with all of the possible members of this type
	static enum LSRegexMembers
	{
		Pattern,
		Substitution,
		Substitute,
		Count,
		MatchResults,
		MatchResult,
		SplitResults,
		SplitResult,
	};
	static enum LSRegexMethods
	{
		SetPattern,
		SetPatternCS,
		SetSubstitution,
		Match,
		Split,
	};

	LSRegexType():LSType("regex")
	{
		// Add each member name to the list, the TypeMember macro automatically uses
		// the name used in the enumeration
		TypeMember(Pattern);
		TypeMember(Substitution);
		TypeMember(Substitute);
		TypeMember(Count);
		TypeMember(MatchResults);
		TypeMember(MatchResult);
		TypeMember(SplitResults);
		TypeMember(SplitResult);

		TypeMethod(SetPattern);
		TypeMethod(SetPatternCS);
		TypeMethod(SetSubstitution);
		TypeMethod(Match);
		TypeMethod(Split);
	};

	~LSRegexType() {}

	bool GetMember(LSOBJECTDATA ObjectData, PCHAR Member, int argc, char *argv[], LSOBJECT &Dest);
	bool GetMethod(LSOBJECTDATA &ObjectData, char *Method, int argc, char *argv[]);

	bool ToString(LSOBJECTDATA ObjectData, PCHAR Destination) {return false;}

	// The following member functions should be modified if you wish to
	// support scripting variables

	void InitVariable(LSOBJECTDATA &ObjectData) {ObjectData.Ptr=new LSRegex;}
	void FreeVariable(LSOBJECTDATA &ObjectData) {LSRegex *pPtr=(LSRegex *)ObjectData.Ptr;delete pPtr;}

	bool FromString(LSOBJECTDATA &ObjectData, PCHAR Source)
	{
		return true;
	}
};